@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009, 2018 Free Software Foundation, Inc.                  */
+/*  Copyright (C) 2009-2019 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -40,6 +40,12 @@
 #include "common.h"
 #include "dwg.h"
 
+// avoid double linkage on windows with unit-testing
+#if defined(BITS_TEST_C) || defined(DECODE_TEST_C)
+#  undef EXPORT
+#  define EXPORT
+#endif
+
 /**
  Structure for DWG-files raw data storage
  */
@@ -52,15 +58,15 @@ typedef struct _bit_chain
   FILE *fh;
   Dwg_Version_Type version;
   Dwg_Version_Type from_version;
+  unsigned char opts;
 } Bit_Chain;
 
 /* Functions for raw data manipulations.
  */
 void bit_advance_position (Bit_Chain *dat, long advance);
-
 unsigned long bit_position (Bit_Chain *dat);
-
 void bit_set_position (Bit_Chain *dat, unsigned long bitpos);
+void bit_reset_chain (Bit_Chain *dat);
 
 BITCODE_B
 bit_read_B (Bit_Chain *dat);
@@ -190,6 +196,9 @@ int bit_check_CRC (Bit_Chain *dat, long unsigned int start_address,
 
 uint16_t bit_write_CRC (Bit_Chain *dat, long unsigned int start_address,
                         const uint16_t seed);
+// object-map only
+uint16_t bit_write_CRC_LE (Bit_Chain *dat, long unsigned int start_address,
+                           const uint16_t seed);
 
 uint16_t bit_calc_CRC (const uint16_t seed, unsigned char *adr, long len);
 
@@ -214,36 +223,37 @@ bit_read_TU (Bit_Chain *restrict dat);
 
 void bit_write_TU (Bit_Chain *restrict dat, BITCODE_TU restrict value);
 
+BITCODE_T bit_read_T (Bit_Chain *restrict dat);
+void bit_write_T (Bit_Chain *restrict dat, BITCODE_T restrict chain);
+
 /* Converts UCS-2 to UTF-8, returning a copy. */
-EXPORT char *bit_convert_TU (BITCODE_TU restrict wstr);
+EXPORT char *bit_convert_TU (BITCODE_TU restrict wstr) ATTRIBUTE_MALLOC;
 
 /** Converts UTF-8 to UCS-2. Returns a copy.
-    Eventually needed by dwg writers (dxf2dwg) */
-EXPORT BITCODE_TU bit_utf8_to_TU (char *restrict str);
+    Needed by dwg importers, writers (e.g. dxf2dwg) */
+EXPORT BITCODE_TU bit_utf8_to_TU (char *restrict str) ATTRIBUTE_MALLOC;
 
 /* compare an ASCII/utf-8 string to a r2007+ name */
 int bit_eq_TU (const char *str, BITCODE_TU restrict wstr);
 
-BITCODE_RL
-bit_read_L (Bit_Chain *dat);
-
+BITCODE_RL bit_read_L (Bit_Chain *dat);
 void bit_write_L (Bit_Chain *dat, BITCODE_RL value);
 
-BITCODE_TIMEBLL
-bit_read_TIMEBLL (Bit_Chain *dat);
+BITCODE_TIMEBLL bit_read_TIMEBLL (Bit_Chain *dat);
+void bit_write_TIMEBLL (Bit_Chain *dat, BITCODE_TIMEBLL date);
 
-void bit_write_TIMEBLL (Bit_Chain *dat, BITCODE_TIMEBLL value);
+BITCODE_TIMERLL bit_read_TIMERLL (Bit_Chain *dat);
+void bit_write_TIMERLL (Bit_Chain *dat, BITCODE_TIMERLL date);
 
 void bit_read_CMC (Bit_Chain *restrict dat, Dwg_Color *restrict color);
-
 void bit_write_CMC (Bit_Chain *restrict dat, Dwg_Color *restrict color);
 
-void bit_read_ENC (Bit_Chain *restrict dat, Dwg_Color *restrict color);
-
-void bit_write_ENC (Bit_Chain *restrict dat, Dwg_Color *restrict color);
+void bit_read_ENC (Bit_Chain *dat, Bit_Chain *hdl_dat, Bit_Chain *str_dat,
+                   Dwg_Color *restrict color);
+void bit_write_ENC (Bit_Chain *dat, Bit_Chain *hdl_dat, Bit_Chain *str_dat,
+                    Dwg_Color *restrict color);
 
 int bit_search_sentinel (Bit_Chain *dat, unsigned char sentinel[16]);
-
 void bit_write_sentinel (Bit_Chain *dat, unsigned char sentinel[16]);
 
 void bit_chain_alloc (Bit_Chain *dat);
@@ -251,6 +261,7 @@ void bit_chain_alloc (Bit_Chain *dat);
 void bit_print (Bit_Chain *dat, long unsigned int size);
 
 void bit_write_bits (Bit_Chain *restrict dat, const char *restrict bits);
+long bit_write_hexbits (Bit_Chain *restrict dat, const char *restrict bytes);
 void bit_print_bits (unsigned char *bits, long unsigned int size);
 void bit_fprint_bits (FILE *fp, unsigned char *bits, long unsigned int size);
 
